/*
  KeePass Password Safe - The Open-Source Password Manager
  Copyright (C) 2003-2009 Dominik Reichl <dominik.reichl@t-online.de>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

using System;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using System.Collections;
using System.IO;
using KeePass.Forms;
using KeePass.UI;
using KeePassLib;
using KeePassLib.Security;
using KeePassLib.Collections;
using KeePass;
using ITanMaster;
using System.Reflection;

namespace ITanMaster
{
    public partial class ITanWizardForm : Form
    {
        private MainForm m_mainForm = null;
        private PwDatabase m_pwDatabase = null;
        private PwGroup m_pgStorage = null;

        private Hashtable m_keys = new Hashtable();
        private int m_offset = 1;
        private bool m_OneTimeTan = false;
        private bool m_DualTokens = false;

        public void InitEx(MainForm mainForm, PwDatabase pwParent, PwGroup pgStorage)
        {
            m_mainForm = mainForm;
            m_pwDatabase = pwParent;
            m_pgStorage = pgStorage;
        }

        public ITanWizardForm()
        {
            InitializeComponent();

            m_cbxParseMode.Items.Add(TokenProvider.InteractiveBrokers.ToString());
            m_cbxParseMode.Items.Add(TokenProvider.Flatex.ToString());
            m_cbxParseMode.Items.Add(TokenProvider.FlatexCH.ToString());
            m_cbxParseMode.Items.Add(TokenProvider.Card.ToString());
            m_cbxParseMode.Items.Add(TokenProvider.Sequence.ToString());
            m_cbxParseMode.SelectedIndex = 0;
            //TODO
            Program.Translation.ApplyTo(this);
        }

        private void OnFormLoad(object sender, EventArgs e)
        {
            Debug.Assert(m_pwDatabase != null); if (m_pwDatabase == null) throw new InvalidOperationException();
            Debug.Assert(m_pgStorage != null); if (m_pgStorage == null) throw new InvalidOperationException();

            GlobalWindowManager.AddWindow(this);

            m_bannerImage.Image = BannerFactory.CreateBanner(m_bannerImage.Width,
                m_bannerImage.Height, BannerStyle.Default,
                Properties.Resources.B48x48_Wizard, Properties.Resources.ITanWizardName,
                Properties.Resources.IBWizardDescription);

            this.Icon = Properties.Resources.Dollar;
            this.Text = Properties.Resources.ITanWizardName;

            if ((m_pgStorage.Name != null) && (m_pgStorage.Name.Length > 0))
                m_lblToGroup.Text += ": " + m_pgStorage.Name + ".";
            else
                m_lblToGroup.Text += ".";

            var versionAttribute = Assembly.GetExecutingAssembly().GetCustomAttributes(typeof(AssemblyFileVersionAttribute), true).FirstOrDefault() as AssemblyFileVersionAttribute;
            lblREV.Text = versionAttribute.Version;

        }

        private void OnBtnOK(object sender, EventArgs e)
        {
            try
            {
                if (!string.IsNullOrEmpty(m_txbITanInput.Text))
                {
                    if (m_keys.Keys.Count == 0) ParseITanText();
                    AddITanKeys();
                }
                CleanUpEx();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Input format error", MessageBoxButtons.OK);
            }
        }

        private void OnBtnCancel(object sender, EventArgs e)
        {
            CleanUpEx();
        }

        private void CleanUpEx()
        {
        }

        private void OnFormClosed(object sender, FormClosedEventArgs e)
        {
            GlobalWindowManager.RemoveWindow(this);
        }

        private TokenProvider m_parseMode = TokenProvider.InteractiveBrokers;
        private void OnParseModeChanged(object sender, EventArgs e)
        {
            ComboBox cbx = (ComboBox)sender;
            m_parseMode = (TokenProvider)Enum.Parse(typeof(TokenProvider), ((string)cbx.SelectedItem));
            switch (m_parseMode)
            {
                case TokenProvider.Sequence:
                    nudTokens.Enabled = true;
                    cbSingleUseTAN.Enabled = true;
                    lblBlockMode1.Visible = false;
                    lblBlockMode2.Visible = false;
                    lblBlockMode3.Visible = false;
                    nudFirstIndex.Visible = true;
                    nudFirstIndex.Enabled = true;
                    nudBlocks.Visible = false;
                    nudLines.Visible = false;
                    nudColumns.Visible = false;
                    nudBlocks.Enabled = false;
                    nudLines.Enabled = false;
                    nudColumns.Enabled = false;
                    break;
                case TokenProvider.Card:
                    nudTokens.Enabled = true;
                    cbSingleUseTAN.Enabled = true;
                    lblBlockMode1.Visible = true;
                    lblBlockMode2.Visible = true;
                    lblBlockMode3.Visible = true;
                    nudFirstIndex.Visible = true;
                    nudFirstIndex.Enabled = true;
                    nudBlocks.Visible = true;
                    nudLines.Visible = true;
                    nudColumns.Visible = true;
                    nudBlocks.Enabled = true;
                    nudLines.Enabled = true;
                    nudColumns.Enabled = true;
                    nudBlocks.Value = nudBlocks.Value == 0 ? 1 : nudBlocks.Value;
                    nudLines.Value = nudLines.Value == 0 ? 1 : nudLines.Value;
                    nudColumns.Value = nudColumns.Value == 0 ? 1 : nudColumns.Value;
                    break;
                case TokenProvider.InteractiveBrokers:
                    nudTokens.Value = 2;
                    nudTokens.Enabled = false;
                    cbSingleUseTAN.Checked = false;
                    cbSingleUseTAN.Enabled = false;
                    lblBlockMode1.Visible = true;
                    lblBlockMode2.Visible = true;
                    lblBlockMode3.Visible = true;
                    nudFirstIndex.Visible = true;
                    nudFirstIndex.Enabled = false;
                    nudFirstIndex.Value = 1;
                    nudBlocks.Visible = true;
                    nudLines.Visible = true;
                    nudColumns.Visible = true;
                    nudBlocks.Enabled = false;
                    nudLines.Enabled = false;
                    nudColumns.Enabled = false;
                    nudBlocks.Value = 2;
                    nudLines.Value = 16;
                    nudColumns.Value = 7;
                    break;
                case TokenProvider.Flatex:
                    nudTokens.Value = 3;
                    nudTokens.Enabled = false;
                    cbSingleUseTAN.Checked = false;
                    cbSingleUseTAN.Enabled = false;
                    lblBlockMode1.Visible = true;
                    lblBlockMode2.Visible = true;
                    lblBlockMode3.Visible = true;
                    nudFirstIndex.Visible = true;
                    nudFirstIndex.Enabled = false;
                    nudFirstIndex.Value = 1;

                    nudBlocks.Visible = true;
                    nudBlocks.Enabled = false;
                    nudBlocks.Value = 1;

                    nudLines.Visible = true;
                    nudLines.Value = 11;
                    nudLines.Enabled = false;

                    nudColumns.Visible = true;
                    nudColumns.Value = 9;
                    nudColumns.Enabled = false;
                    break;
                case TokenProvider.FlatexCH:
                    nudTokens.Value = 1;
                    nudTokens.Enabled = false;
                    cbSingleUseTAN.Checked = false;
                    cbSingleUseTAN.Enabled = false;
                    lblBlockMode1.Visible = true;
                    lblBlockMode2.Visible = true;
                    lblBlockMode3.Visible = true;
                    nudFirstIndex.Visible = true;
                    nudFirstIndex.Enabled = false;
                    nudFirstIndex.Value = 1;

                    nudBlocks.Visible = true;
                    nudBlocks.Enabled = false;
                    nudBlocks.Value = 1;

                    nudLines.Visible = true;
                    nudLines.Value = 10;
                    nudLines.Enabled = false;

                    nudColumns.Visible = true;
                    nudColumns.Value = 8;
                    nudColumns.Enabled = false;
                    break;
                default:
                    throw new NotImplementedException("Provider type " + m_cbxParseMode + " not implemented");
            }
        }

        private string m_originalInput = "";
        private void btnTestGen_Click(object sender, EventArgs e)
        {
            m_originalInput = m_txbITanInput.Text;
            m_txbITanInput.Text = "";
            m_btnOK.Enabled = false;
            StringBuilder sb = new StringBuilder();
            string vals = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            int maxPos = vals.Length - 1;
            Hashtable _keys = new Hashtable();
            Random gen = new Random();
            int blocks = (int)nudBlocks.Value;
            int lines = (int)nudLines.Value;
            int columns = (int)nudColumns.Value;
            int maxidx = m_offset + m_keys.Count - 1;

            int max = (int)(blocks * lines * columns);

            if (max > 10000) return;

            for (int loop = 1; loop <= max; ++loop)
            {
                bool inserted = false;
                while (!inserted)
                {
                    int k1 = gen.Next(maxPos);
                    int k2 = gen.Next(maxPos);
                    int k3 = gen.Next(maxPos);
                    while (k2 == k1 || k3 == k2 || k1 == k3)
                    {
                        k2 = gen.Next(maxPos);
                        k3 = gen.Next(maxPos);
                    }
                    string k = vals.Substring(k1, 1) + vals.Substring(k2, 1) + vals.Substring(k3, 1);
                    if (!_keys.ContainsValue(k))
                    {
                        inserted = true;
                        _keys.Add(loop, k);
                    }
                }
            }

            for (int page = 0; page < blocks; ++page)
            {
                for (int line = 0; line < lines; ++line)
                {
                    for (int col = 0; col < columns; ++col)
                    {
                        int pos = (lines * columns * page) + (columns * line) + col + 1;
                        sb.Append(_keys[pos] + " ");
                    }
                    sb.Append(Environment.NewLine);
                }
                sb.Append(Environment.NewLine);
            }

            m_txbITanInput.Text = sb.ToString();

        }

        private void OnBtnParse(object sender, EventArgs e)
        {
            try
            {
                ParseITanText();
                DisplayKeyCard();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Input format error", MessageBoxButtons.OK);
            }
        }

        private void ParseITanText()
        {
            switch (m_parseMode)
            {
                case TokenProvider.Sequence:
                case TokenProvider.Flatex:
                case TokenProvider.FlatexCH:
                    ParseSequential();
                    break;
                case TokenProvider.InteractiveBrokers:
                case TokenProvider.Card:
                    ParseCard();
                    break;
            }
        }

        private void ParseSequential()
        {
            m_keys.Clear();
            StringReader sr = new StringReader(m_txbITanInput.Text);
            try
            {
                int idx = m_offset - 1;
                string inp = "";
                while (true)
                {
                    inp = sr.ReadLine();
                    if (string.IsNullOrEmpty(inp)) break;
                    inp = inp.Trim();
                    string[] vals = inp.Split(' ');
                    for (int col = 0; col < vals.Length; ++col)
                    {
                        ++idx;
                        m_keys.Add(idx, vals[col]);
                    }
                }
            }
            finally
            {
                sr.Close();
            }
        }

        private void ParseCard()
        {
            m_keys.Clear();
            int blocks = (int)nudBlocks.Value;
            int lines = (int)nudLines.Value;
            int columns = (int)nudColumns.Value;
            StringReader sr = new StringReader(m_txbITanInput.Text);
            String err = Properties.Resources.ITanCardModeInputError;
            try
            {
                while (true)
                {
                    for (int block = 0; block < blocks; ++block)
                    {
                        for (int line = 0; line < lines; ++line)
                        {
                            string inp = "";
                            while (string.IsNullOrEmpty(inp = sr.ReadLine()))
                            {
                                if (inp == null) throw new Exception(err);
                            }
                            inp = inp.Trim();
                            string[] vals = inp.Split(' ');

                            if (vals.Length != columns) throw new Exception(err);

                            for (int col = 0; col < columns; ++col)
                            {
                                int idx = m_offset + (lines * columns * block) + (lines * col) + line;
                                m_keys.Add(idx, vals[col]);
                            }
                        }
                    }
                    break;
                }
            }
            finally
            {
                sr.Close();
            }
        }

        private void DisplayKeyCard()
        {
            StringBuilder sb = new StringBuilder();
            int blocks = (int)nudBlocks.Value;
            int lines = (int)nudLines.Value;
            int columns = (int)nudColumns.Value;
            int maxidx = m_offset + m_keys.Count - 1;
            int idx = m_offset - 1;
            switch (m_parseMode)
            {
                case TokenProvider.Sequence:
                    int count = m_keys.Count;
                    columns = 8;
                    lines = count / columns;
                    if (count - (lines * columns) > 0) ++lines;
                    blocks = 1;
                    for (int line = 0; line < lines; ++line)
                    {
                        for (int col = 0; col < columns; ++col)
                        {
                            idx = m_offset + columns * line + col;
                            if (idx > maxidx) break;
                            sb.AppendFormat("{0:D3} {1} ", idx, m_keys[idx]);
                        }
                        sb.Append(Environment.NewLine);
                        if (idx > maxidx) break;
                    }
                    sb.Append(Environment.NewLine);
                    break;
                case TokenProvider.Flatex:
                    //header
                    sb.Append("  ");
                    for (int col = 0; col < columns; ++col)
                    {
                        sb.AppendFormat("   {0:D1}   ", col + 1);
                    }
                    sb.Append(Environment.NewLine);

                    for (int line = 0; line < lines; ++line)
                    {
                        sb.AppendFormat("{0} ", Enum.GetName(typeof(FlatexLines), line));
                        for (int col = 0; col < columns; ++col)
                        {
                            idx = m_offset + (int)((line * columns)) + col;
                            if (idx > maxidx) break;
                            sb.AppendFormat("{0:D2}:{1:4} ", idx, m_keys[idx]);
                        }
                        sb.Append(Environment.NewLine);
                        if (idx > maxidx) break;
                    }
                    sb.Append(Environment.NewLine);
                    break;
                case TokenProvider.FlatexCH:
                    //header
                    sb.Append("      ");
                    for (int col = 0; col < columns; ++col)
                    {
                        sb.Append(Enum.GetName(typeof(FlatexCHHeader), col) + "      ");
                    }
                    sb.Append(Environment.NewLine);

                    for (int line = 0; line < lines; ++line)
                    {
                        sb.AppendFormat("{0:D2} ", line+1);
                        for (int col = 0; col < columns; ++col)
                        {
                            idx = m_offset + (int)((line * columns)) + col;
                            if (idx > maxidx) break;
                            sb.AppendFormat("{0:D2}:{1:4} ", idx, m_keys[idx]);
                        }
                        sb.Append(Environment.NewLine);
                        if (idx > maxidx) break;
                    }
                    sb.Append(Environment.NewLine);
                    break;
                case TokenProvider.Card:
                case TokenProvider.InteractiveBrokers:
                default:
                    for (int block = 0; block < blocks; ++block)
                    {
                        for (int line = 0; line < lines; ++line)
                        {
                            for (int col = 0; col < columns; ++col)
                            {
                                idx = m_offset + (int)((lines * columns) * block) + (lines * col) + line;
                                if (idx > maxidx) break;
                                sb.AppendFormat("{0:D3} {1} ", idx, m_keys[idx]);
                            }
                            sb.Append(Environment.NewLine);
                            if (idx > maxidx) break;
                        }
                        sb.Append(Environment.NewLine);
                        if (idx > maxidx) break;
                    }
                    break;
            }
            m_txbITanInput.Text = sb.ToString();
        }

        private void AddITanKeys()
        {
            for (int index = m_offset; index <= m_keys.Count + m_offset - 1; ++index)
            {
                AddITanKey((string)m_keys[index], index);
            }
        }

        private void AddITanKey(string strITanToken, int intITanIndex)
        {
            if (strITanToken.Length == 0) return;
            if (intITanIndex < m_offset || intITanIndex > m_offset + m_keys.Count - 1) return;

            PwEntry pe = new PwEntry(true, true);

            string tanIndex = intITanIndex.ToString();

            if (m_OneTimeTan)
            {
                pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                    m_pwDatabase.MemoryProtection.ProtectTitle, PwDefs.TanTitle));
            }
            else
            {
                pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                    m_pwDatabase.MemoryProtection.ProtectTitle, ITanMasterDefs.ITanTitleEntries));
            }
            pe.Strings.Set(PwDefs.UserNameField, new ProtectedString(
                m_pwDatabase.MemoryProtection.ProtectUserName, tanIndex));

            pe.Strings.Set(PwDefs.PasswordField, new ProtectedString(
                m_pwDatabase.MemoryProtection.ProtectPassword, strITanToken));

            var groupEntries = from e in m_pgStorage.GetEntries(false)
                               where e.Strings.Get(PwDefs.UserNameField).ReadString() == tanIndex
                               select e;
            int count = groupEntries.Count();
            if (count == 0)
            {
                m_pgStorage.AddEntry(pe, true);
            }
            else
            {
                Debug.Assert(count == 1, count + " records with identical iTan index");
                foreach (var entry in groupEntries)
                {
                    entry.Strings.Set(PwDefs.PasswordField,
                        new ProtectedString(m_pwDatabase.MemoryProtection.ProtectPassword, strITanToken));
                    pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                        m_pwDatabase.MemoryProtection.ProtectTitle, ITanMasterDefs.ITanTitleEntries));
                    entry.Touch(true);
                }
            }
        }

        private void OnInitGroup_Click(object sender, EventArgs e)
        {
            try
            {
                PwGroup ITanGroup = m_pgStorage;

                if (ITanGroup.Entries.Count()>0)
                {
                    DialogResult dr = MessageBox.Show("Group not empty - do you want to continue?", "Entries present", MessageBoxButtons.OKCancel);
                    if (dr != DialogResult.OK) return;
                }

                var entries = from entry in m_pgStorage.GetEntries(false)
                              where entry.Strings.Get(PwDefs.TitleField).ReadString().StartsWith(ITanMasterDefs.ITanFirstEntry)
                              select entry;

                if (entries.Count() == 0)
                {
                    PwEntry pe = new PwEntry(true, true);
                    pe.IconId = PwIcon.Money;

                    pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                        m_pwDatabase.MemoryProtection.ProtectTitle, ITanMasterDefs.ITanFirstEntry));

                    pe.Strings.Set(PwDefs.PasswordField, new ProtectedString(
                        m_pwDatabase.MemoryProtection.ProtectPassword, "TODO: Edit Password"));

                    pe.Strings.Set(PwDefs.UserNameField, new ProtectedString(
                        m_pwDatabase.MemoryProtection.ProtectUserName, "TODO: Edit Account"));
                    if (m_parseMode == TokenProvider.InteractiveBrokers)
                    {
                        //tb 06012012 pe.AutoType.Set("TODO: Edit Window Reference", "{UserName}{TAB}{Password}{ENTER}");
                        pe.AutoType.Add(new KeePassLib.Collections.AutoTypeAssociation("TODO: Edit Window Reference", "{UserName}{TAB}{Password}{ENTER}"));
                    }
                    else
                    {
                        //tb 06012012 pe.AutoType.Set("*Login*", "{UserName}{TAB}{Password}{ENTER}");
                        pe.AutoType.Add(new KeePassLib.Collections.AutoTypeAssociation("*Login*", "{UserName}{TAB}{Password}{ENTER}"));
                    }
                    pe.Touch(true);
                    ITanGroup.AddEntry(pe, true);
                }
                else
                {
                    string msg = "First iTAN key item already stored in selected group. "
                        + " Do you want to continue?)";
                    DialogResult dr = MessageBox.Show(msg, "Key Item already stored?", MessageBoxButtons.YesNo);
                    if (dr != DialogResult.Yes) return;
                }

                entries = from entry in m_pgStorage.GetEntries(false)
                          where entry.Strings.Get(PwDefs.TitleField).ReadString().StartsWith(ITanMasterDefs.ITanSecondEntry)
                          select entry;

                if (entries.Count() == 0)
                {
                    PwEntry pe = new PwEntry(true, true);
                    pe.IconId = PwIcon.Money;

                    pe.Strings.Set(PwDefs.PasswordField, new ProtectedString(
                        m_pwDatabase.MemoryProtection.ProtectPassword, ""));
                    pe.Strings.Set(PwDefs.NotesField, new ProtectedString(
                        m_pwDatabase.MemoryProtection.ProtectNotes,
                        "Do not modify Title field - required for ITan token injection"));

                    switch (m_parseMode)
                    {
                        case TokenProvider.InteractiveBrokers:
                            pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectTitle, ITanMasterDefs.ITanSecondEntry + m_parseMode));
                            pe.Strings.Set(PwDefs.UserNameField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectUserName, "IB Token Access"));
                            //tb 06012012 pe.AutoType.Set("Security Code Card Authentication", "{Enter}");
                            pe.AutoType.Add(new AutoTypeAssociation("Security Code Card Authentication", "{Enter}"));
                            break;
                        case TokenProvider.Flatex:
                            pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectTitle, ITanMasterDefs.ITanSecondEntry + m_parseMode));
                            pe.Strings.Set(PwDefs.UserNameField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectUserName, "Flatex Token Access"));
                            //tb 06012012 pe.AutoType.Set("Security Code Card Authentication", "{Enter}");
                            pe.AutoType.Add(new AutoTypeAssociation("Security Code Card Authentication", "{Enter}"));
                            break;
                        case TokenProvider.FlatexCH:
                            pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectTitle, ITanMasterDefs.ITanSecondEntry + m_parseMode));
                            pe.Strings.Set(PwDefs.UserNameField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectUserName, "FlatexCH Token Access"));
                            pe.AutoType.Add(new AutoTypeAssociation("Security Code Card Authentication", "{Enter}"));
                            break;
                        case TokenProvider.Sequence:
                        case TokenProvider.Card:
                        default:
                            pe.Strings.Set(PwDefs.TitleField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectTitle, ITanMasterDefs.ITanSecondEntry + (nudTokens.Value)));
                            pe.Strings.Set(PwDefs.UserNameField, new ProtectedString(
                                m_pwDatabase.MemoryProtection.ProtectUserName, "TODO: edit Autokey sequence"));
                            //tb 06012012 pe.AutoType.Set("TODO: edit Autokey sequence", "");
                            pe.AutoType.Add(new AutoTypeAssociation("TODO: edit Autokey sequence", ""));
                            break;
                    }
                    pe.Touch(true);
                    ITanGroup.AddEntry(pe, true);
                }
                else
                {
                    string msg = "Second iTAN key item already stored in selected group. "
                        + " Do you want to continue?)";
                    DialogResult dr = MessageBox.Show(msg, "Key Item 2 already stored?", MessageBoxButtons.YesNo);
                    if (dr != DialogResult.Yes) return;
                }
            }
            finally
            {
                m_mainForm.UpdateUI(false, null, true, null, true, null, true);
            }
        }

        private void OnOffsetChangedChanged(object sender, EventArgs e)
        {
            NumericUpDown nud = (NumericUpDown)sender;
            if (nud.Value < 1) nud.Value = 1;
            m_offset = (int)nud.Value;
        }

        private void OnDualTokens_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = (CheckBox)sender;
            m_DualTokens = cb.Checked;
        }

        private void OnSingleUseTAN_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = (CheckBox)sender;
            m_OneTimeTan = cb.Checked;
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            m_btnOK.Enabled = true;
            if (string.IsNullOrEmpty(m_originalInput))
            {
                m_txbITanInput.Text = "";
            }
            else
            {
                m_txbITanInput.Text = m_originalInput;
                m_originalInput = "";
            }
        }
    }
}
